#!/usr/bin/python
# -*- coding: UTF-8 -*-
from dli.column import Column
from dli.row import Row


class ResultSetIterator(object):
    def __init__(self, result_set):
        self._result_set = result_set

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        try:
            ret = self._result_set.next()
            if ret is not True:
                raise StopIteration()

            return self._result_set.fetch()
        except IndexError:
            raise StopIteration()


class ResultSet(object):
    def __init__(self, job, rows=None, schema=None):
        self._job = job
        self._rows = []
        self._schema = []
        self._row_idx = 0
        self._local_idx = 0
        self._row_count = 0
        self._current_page = 1
        self._current_row = None

        if rows is not None:
            self._row_count = len(rows)
            self._rows = rows
        else:
            self._get_result()

    def __iter__(self):
        return ResultSetIterator(self)

    def next(self):
        if self._row_idx >= self._row_count:
            return False

        if self._local_idx >= len(self._rows):
            self._get_result()
            self._local_idx = 0

        self._current_row = self._rows[self._local_idx]
        self._row_idx += 1
        self._local_idx += 1
        return True

    def fetch(self):
        return self._current_row

    def _get_result(self):
        job_result = self._job.get_result_page(self._current_page)

        if self._current_page == 1:
            self._schema = []
            for col in job_result.schema:
                for col_name, col_type in col.items():
                    self._schema.append(Column(col_name, col_type))

            self._row_count = job_result.row_count

        self._rows = []
        for row in job_result.rows:
            self._rows.append(Row(self._schema, row))

        self._current_page += 1

    @property
    def row_count(self):
        return self._row_count
